# This file was generated by liblab | https://liblab.com/

from enum import Enum

from .utils.base_model import BaseModel
from .utils.json_map import JsonMap


class WatermarkCreatePosition(Enum):
    """An enumeration representing different categories.

    :cvar UPPERRIGHT: "upper_right"
    :vartype UPPERRIGHT: str
    :cvar UPPERLEFT: "upper_left"
    :vartype UPPERLEFT: str
    :cvar LOWERRIGHT: "lower_right"
    :vartype LOWERRIGHT: str
    :cvar LOWERLEFT: "lower_left"
    :vartype LOWERLEFT: str
    :cvar CENTER: "center"
    :vartype CENTER: str
    """

    UPPERRIGHT = "upper_right"
    UPPERLEFT = "upper_left"
    LOWERRIGHT = "lower_right"
    LOWERLEFT = "lower_left"
    CENTER = "center"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, WatermarkCreatePosition._member_map_.values())
        )


@JsonMap({})
class WatermarkCreate(BaseModel):
    """WatermarkCreate

    :param file: The image to be used as watermark. The image will not be resized after uploading. Maximum filesize is 2.0 MB. File must be a valid PNG image.
    :type file: bytes
    :param opacity: The watermark's translucency can be adjusted on a scale from 0.0 to 1.0. A setting of 0.0 renders the watermark entirely transparent, while a setting of 1.0 makes it fully opaque. If the watermark originally possesses a degree of semi-transparency, selecting a value of 1.0 won't result in total opacity.
    :type opacity: float
    :param padding: Empty padding space between the neighboring borders (defined by position) of the video and the watermark. `0.0` means no padding at all, while 1.0 means full padding across the video's width or height.
    :type padding: float
    :param position: Position where the watermark will be placed inside the video. Note that when the watermark is positioned at the center, no `padding` will be used.
    :type position: WatermarkCreatePosition
    :param scale: Scale the watermark to (`scale` * height) of the video, while preserving its display aspect ratio., defaults to None
    :type scale: float, optional
    :param title: Title of the watermark.
    :type title: str
    """

    def __init__(
        self,
        file: bytes,
        opacity: float,
        padding: float,
        position: WatermarkCreatePosition,
        title: str,
        scale: float = None,
    ):
        """WatermarkCreate

        :param file: The image to be used as watermark. The image will not be resized after uploading. Maximum filesize is 2.0 MB. File must be a valid PNG image.
        :type file: bytes
        :param opacity: The watermark's translucency can be adjusted on a scale from 0.0 to 1.0. A setting of 0.0 renders the watermark entirely transparent, while a setting of 1.0 makes it fully opaque. If the watermark originally possesses a degree of semi-transparency, selecting a value of 1.0 won't result in total opacity.
        :type opacity: float
        :param padding: Empty padding space between the neighboring borders (defined by position) of the video and the watermark. `0.0` means no padding at all, while 1.0 means full padding across the video's width or height.
        :type padding: float
        :param position: Position where the watermark will be placed inside the video. Note that when the watermark is positioned at the center, no `padding` will be used.
        :type position: WatermarkCreatePosition
        :param scale: Scale the watermark to (`scale` * height) of the video, while preserving its display aspect ratio., defaults to None
        :type scale: float, optional
        :param title: Title of the watermark.
        :type title: str
        """
        self.file = file
        self.opacity = self._define_number("opacity", opacity, ge=0, le=1)
        self.padding = self._define_number("padding", padding, ge=0, le=1)
        self.position = self._enum_matching(
            position, WatermarkCreatePosition.list(), "position"
        )
        if scale is not None:
            self.scale = self._define_number("scale", scale, ge=0, le=1)
        self.title = self._define_str("title", title, min_length=1, max_length=64)
