# This file was generated by liblab | https://liblab.com/

from ..models import Project, ProjectCreate, ProjectEdit, ProjectList
from ..models.utils.cast_models import cast_models
from ..net.transport.serializer import Serializer
from .utils.base_service import BaseService
from .utils.validator import Validator


class ProjectService(BaseService):

    @cast_models
    def list_projects(self) -> ProjectList:
        """List available projects.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: ProjectList
        """

        serialized_request = (
            Serializer(f"{self.base_url}/api/v1/project", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return ProjectList._unmap(response)

    @cast_models
    def create_project(self, request_body: ProjectCreate) -> Project:
        """Create a new project. A default player will be created with it.

        :param request_body: The request body.
        :type request_body: ProjectCreate
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: Project
        """

        Validator(ProjectCreate).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/api/v1/project", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return Project._unmap(response)

    @cast_models
    def get_project(self, project_id: int) -> Project:
        """Get settings of a single project by ID.

        :param project_id: The ID of the project.
        :type project_id: int
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: Project
        """

        Validator(int).min(1).max(9223372036854776000).validate(project_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/api/v1/project/{{project_id}}",
                self.get_default_headers(),
            )
            .add_path("project_id", project_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return Project._unmap(response)

    @cast_models
    def edit_project(self, request_body: ProjectEdit, project_id: int) -> Project:
        """Edit settings that apply to a single project.

        :param request_body: The request body.
        :type request_body: ProjectEdit
        :param project_id: The ID of the project.
        :type project_id: int
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: OK
        :rtype: Project
        """

        Validator(ProjectEdit).validate(request_body)
        Validator(int).min(1).max(9223372036854776000).validate(project_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/api/v1/project/{{project_id}}",
                self.get_default_headers(),
            )
            .add_path("project_id", project_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return Project._unmap(response)
